# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime
import time

from driverng.protocol import VoidProtocol


class AlereProtocol(VoidProtocol):
    ACK = b"""<?xml version="1.0" encoding="utf-8"?><ACK.R01><HDR><HDR.control_id V="%(control_id)s"/><HDR.version_id V="%(version_id)s"/><HDR.creation_dttm V="%(creation_dttm)s"/></HDR><ACK><ACK.type_cd V="%(type_cd)s"/><ACK.ack_control_id V="%(ack_control_id)s"/></ACK></ACK.R01>"""
    REQ = b"""<?xml version="1.0" encoding="utf-8"?><REQ.R01><HDR><HDR.control_id V="%(control_id)s"/><HDR.version_id V="%(version_id)s"/><HDR.creation_dttm V="%(creation_dttm)s"/></HDR><REQ><REQ.request_cd V="%(request_cd)s"/></REQ></REQ.R01>"""

    def get_ack_message(self, payload, date=None, control_id=None):
        date = date or datetime.datetime.now()
        control_id = b"%d" % int(time.time() * 1000) if not control_id else control_id
        context = {
            b"control_id": control_id,
            b"version_id": b"POCT1",
            b"creation_dttm": date.strftime("%Y-%m-%dT%H:%M:%S%z").encode(),
            b"type_cd": b"AA",
            b"ack_control_id": control_id,
        }
        return self.ACK % context

    def get_req_message(self, payload, date=None, control_id=None):
        date = date or datetime.datetime.now()
        control_id = b"%d" % int(time.time() * 1000) if not control_id else control_id
        context = {
            b"control_id": control_id,
            b"version_id": b"POCT1",
            b"creation_dttm": date.strftime("%Y-%m-%dT%H:%M:%S%z").encode(),
            b"request_cd": b"ROBS",
        }
        return self.REQ % context

    def recv_trames(self):
        data = self.read(size=4096)
        self.write(self.get_ack_message(self))
        data_lines = data.splitlines()
        for line in data_lines:
            if b"<DST.R01>" in line:
                self.write(self.get_req_message(self))
                break
        to_replace = [b"<A>", b"<lf>"]
        for i in to_replace:
            data = data.replace(i, b"\r")
        return data
